import psutil

class cpu():
    def percent():
        return(psutil.cpu_percent(percpu=True))

    def count():
        return(psutil.cpu_count(logical=False), psutil.cpu_count())

class ram():
    def virtual():
        return(psutil.virtual_memory())

    def swap():
        return(psutil.swap_memory())

class disks():
    def partitions():
        return(psutil.disk_partitions())

    def usage():
        return(psutil.disk_usage('/'))

class pids():
    def pids():
        PidList = psutil.pids()
        processes = []
        for q in PidList:
            w = psutil.Process(q)
            processes.append(w)
        return(processes)
    
    def ids():
        return(psutil.pids())

    def lookup(ID):
        return(psutil.Process(ID))

    def get():
        psutil.test()

class other():
    def battery():
        return(psutil.sensors_battery())

    def users():
        return(psutil.users())

    def time():
        return(psutil.boot_time())



# pyautoflow
# Copyright (C) 2023 Daniel Drury
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

import keyboard
import mouse
import numpy as np
import cv2
import pyautogui
import threading

recording_thread = None
recording_stopping = False

def recording_thread_func(filename):
    screen_width, screen_height = pyautogui.size()
    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    out = cv2.VideoWriter(filename, fourcc, 20.0, (screen_width, screen_height))
    while True:
        out.write(cv2.cvtColor(np.array(pyautogui.screenshot()), cv2.COLOR_BGR2RGB))
        if recording_stopping:
            out.release()
            break

def screenshot():
    img = pyautogui.screenshot()
    return(img)

def screenshot_area(x1, y1, x2, y2):
    img = pyautogui.screenshot(region=(x1,y1,x2,y2))
    return(img)

def save_img(img, filename):
    if (filename[-4:] != ".png") and (filename[-4:] != ".jpg"):
        raise ValueError("Must be a .png or .jpg file extension.")
    image = cv2.cvtColor(np.array(img), cv2.COLOR_RGB2BGR)
    cv2.imwrite(filename, image)

def start_recording(filename):
    if filename[-4:] != ".mp4":
        raise ValueError("Must be a .mp4 file extension.")
    global recording_thread
    global recording_stopping
    recording_stopping = False
    recording_thread = threading.Thread(target = recording_thread_func, args = (filename,))
    recording_thread.start()    

def stop_recording():
    global recording_stopping
    recording_stopping = True



# pyautoflow
# Copyright (C) 2023 Daniel Drury
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

import keyboard
import mouse
import time

timer_start_time = time.time()

def press(key):
    keyboard.press_and_release(key)

def press_and_hold(key, duration):
    keyboard.press(key)
    time.sleep(duration)
    keyboard.release(key)

def type(text):
    keyboard.send(text)

def type_custom(text, delay):
    for q in text:
        press(q)
        time.sleep(delay)

def set_cursor(x, y):
    mouse.move(x, y)

def set_cursor_rel(x, y):
    mouse.move(x, y, False)

def move_cursor(x, y, time):
    mouse.move(x, y, duration=time)

def move_cursor_rel(x, y, time):
    mouse.move(x, y, False, duration=time)

def click():
    mouse.click()

def right_click():
    mouse.right_click()

def click_custom(count, delay, button=0):
    if button == 0: # Left button
        for q in range(count):
            click()
            time.sleep(delay)       
    if button == 1: # Right button
        for q in range(count):
            right_click()
            time.sleep(delay)

def press_mouse(button=0):
    if button == 0: # Left button
        mouse.press('left')
    if button == 1: # Right button
        mouse.press('right')

def release_mouse(button=0):
    if button == 0: # Left button
        mouse.release('left')
    if button == 1: # Right button
        mouse.release('right')

def drag(x1, y1, x2, y2, duration):
    set_cursor(x1, y1)
    press_mouse()
    move_cursor(x2, y2, duration)
    release_cursor()

def reset_timer():
    global timer_start_time
    timer_start_time = time.time()

def get_timer():
    global timer_start_time
    timer = time.time() - timer_start_time
    return timer

